% Hi Jeff,
% Here's a bit of code that calculates the single, 3D axis of rotation for a rigid object from time t to t+1
% input: unit vectors of a rigid object in space (in this case for at least two timesteps, but for two different objects at the same time is of course similar) 
% output: a sinlge axis of rotation from time t to t+1
% This brute force approach is equivalent to rotation calculations via quaternions, but to me this is a bit more intuitive (be it computationally slower).
%
% Euler's theorem of rotations states something along the lines of (don't quote me on this): 
% The rotation of a rigid object from one orientation (angular position) to another can be described by a single vector (the equivalent angle axis representation (EAAR, or K), or euler pole). 
% The direction (the unit vector of K, Khat) represents the axis of rotation, and the magnitude (theta) indicates how much the object should rotate to get from the orientation at t to the orientation at t+1. 
% I've commented each step below, let me know if things are unclear, we can always skype about this if that would help. 
% I'm 'gewoonivo' on Skype.
% Cheers,
% Ivo

EAAR=NaN(L,3);                                                              % array for the equivalent angle-axis representation
for i=1:L-1;                                                                % in a for-loop, so you can calculated sequential rotations of an object
    % the object is represented by a rigid orthogonal frame(say the bone's AP, ML and DV axes, all described in, say, the Newtonian frame, N (= Inertial frame = room frame = global frame) 
    % for time i: ObjUVs(i,1:9)=[APx APy APz MLx MLy MLz DVx DVy DVz];
    NtoO=[ObjUVs(i,1:3);ObjUVs(i,4:6);ObjUVs(i,7:9)];                      % rotation matrix from Newtonian to Object reference frame, constructed simply from the Object unit vectors (UVs).
    OtoN=NtoO';                                                             % rotation matrix from Object to Newtonian reference frame, just in case you want to reverse the rotation.
    for n=1:3;                                                              % for matrix rotation reasons, a step-wise rotation is implemented.                                          
        ObjUVsP(1,n*3-2:n*3)=(NtoO * ObjUVs(i+1,n*3-2:n*3).').';            % ObjectUVsP(P for Prime) is Objt+1 described in Objt coordinates
    end
    R_Ktheta=[ObjUVsP(1,1:3);ObjUVsP(1,4:6);ObjUVsP(1,7:9)];                % General orientation of Obj_t+1 relative to Obj_t = R_Ktheta = 'the rotation matrix from t to t+1'
    Kmag=(180/pi)*acos((R_Ktheta(1,1)+R_Ktheta(2,2)+R_Ktheta(3,3)-1)/2)/timestep;     % EAAR magnitude [deg/sec]. See accompanying Craig, Robotics pdf for details / theoretical background
    Khat_O(1,1:3)=-1*(1/(2*sin(Kmag*timestep*pi/180)))*[R_Ktheta(3,2)-R_Ktheta(2,3);R_Ktheta(1,3)-R_Ktheta(3,1);R_Ktheta(2,1)-R_Ktheta(1,2)]; % Khat expressed in Obj at i. -1 to obtain the correct solution
	Khat_N=(OtoN * Khat_O.').';                                             % Khat expressed in N at i
    Kvec_N=Khat_N * Kmag;
    Kvec_O=Khat_O * Kmag;% Kvector, expressed in N: direction * magnitude
    EAAR(i,:)=Kvec_O; % you can write the output EAAR to an array for subsequent processing of continuing rotations.
    % 3D rotations can be projected (using the dot product, see below) on any axis of interest, to represent the amount of rotation about that axis. 
    % If you're interested in rotation components about any of the object's own (bone) axes (AP, ML, or DV), then there are three ways to Rome:
    % I: you can just express Kvec in O, and read out the components:
    Kvec_O___I=(NtoO * Kvec_N.').';
    % II: you can use the Khat expressed in Object frame to multiply with the magnitude, since you already calculated Khat_O above:
    Kvec_O__II=Khat_O * Kmag;
    % II: you can compile the EAAR in the bone frame from separate projections on each of the axes:
%     Kvec_O_AP=dot(Kvec_N,ObjectUVs(i,1:3));                                 % Note that since both vectors in this dot product are unit vectors, no correction for their length is needed)
%     Kvec_O_ML=dot(Kvec_N,ObjectUVs(i,4:6));
%     Kvec_O_DV=dot(Kvec_N,ObjectUVs(i,7:9));
%     Kvec_O_III=[Kvec_O_AP Kvec_O_ML Kvec_O_DV];
ase(i,:)=Khat_O;
end
%%
RPYP=NaN(L,3);                                                            % R.A. head position
RPYP(1,:)=[0 0 0];                                                 % integration constant: using CMpos at first frame. will only lead to shift of movement rel to head (of frame center)
for i=1:L-1;
    RPYP(i+1,1:3)=RPYP(i,:)+EAAR(i,1:3)*timestep;                        % CoM 3D velocity in m/s, expressed in the GCS
end